@@ -15,7 +15,8 @@ import numpy as np
 import pylab
 
 
-from imfun import fseq, fnmap, lib, leica, bwmorph
+from imfun import fseq, fnmap, lib, leica, bwmorph, atrous
+from imfun import filt
 from imfun import ui as ifui
 import glob as Glob
 
@@ -25,15 +26,15 @@ from matplotlib.figure import Figure
 #from matplotlib.backends.backend_wx import NavigationToolbar2Wx
 from matplotlib.backends.backend_wx import NavigationToolbar2Wx as NToolbar
 
-from enthought.traits.api import *
-from enthought.traits.ui.api import *
+from traits.api import *
+from traitsui.api import *
 
 
-from enthought.traits.ui.menu \
+from traitsui.menu \
      import Action, CloseAction, Menu, MenuBar, OKCancelButtons, Separator
 
-from enthought.traits.ui.wx.editor import Editor
-from enthought.traits.ui.basic_editor_factory import BasicEditorFactory
+from traitsui.wx.editor import Editor
+from traitsui.basic_editor_factory import BasicEditorFactory
 
 
 class _MPLFigureEditor(Editor):
@@ -134,10 +135,10 @@ class GWOpts(HasTraits):
             if self.tmedian_k < 3:
                 pwfn = fn1
             else:
-                pwfn = lib.flcompose2(fn1,
+                pwfn = lib.flcompose(fn1,
                                       medfilt_fn(self.tmedian_k))
             seq1 = seq.pw_transform(pwfn)
-            seq1.fns = [partial(fseq.gauss_blur, size=self.gauss_size)]
+            seq1.fns = [partial(filt.gauss_blur, size=self.gauss_size)]
             arr = seq1.as3darray()     # new array with spatially-smoothed data
             sds = float(np.std(arr))   # standard deviation in all data
             binarr = arr > sds*self.sigma_thresh
@@ -172,25 +173,33 @@ class GWOpts(HasTraits):
 
 class FrameSequenceOpts(HasTraits):
     mfilt7 = lambda v: signal.medfilt(v,7)
+    tmedian_k = Enum([5]+range(1,13,2), label='median filter kernel')
+    #normL = Int(250, label="N baseline frames")
     fw_presets = {
         '1. Do nothing' : [],
-        '2. Gauss blur' : [fseq.gauss_blur],
+        '2. Gauss blur' : [filt.gauss_blur],
         '3. Median filter' : [lambda v:signal.medfilt(v,3)]
         }
     pw_presets = {
-        '1. Do nothing' : lambda x:x,
-        '2. Norm to SD' : lambda x: x/np.std(x),
-        '3. DF/F' : lib.DFoF,
-        '4. DF/SD' : lib.DFoSD,
-        '5. Med. filter ' : mfilt7,
-        '6. Med. filter + DF/F': lib.flcompose(mfilt7,lib.DFoF),
-        '7. Med. filter + DF/SD':lib.flcompose(mfilt7,lib.DFoSD),
+        '01. Do nothing' : lambda x:x,
+        '02. Norm to SD' : lambda x: x/np.std(x),
+        '03. DF/F' : lib.DFoF,
+        '04. DF/sigma' : lib.DFoSD,
+	'05. DF/F with detrend': atrous.DFoF,
+	'06. DF/sigma with detrend': atrous.DFoSD,	
+        '07. Med. filter ' : mfilt7,
+        '08. Med. filter -> DF/F': lib.flcompose(mfilt7,lib.DFoF),
+        '09. DF/F -> Med. filter': lib.flcompose(lib.DFoF, mfilt7),
+        '10. Med. filter -> DF/SD':lib.flcompose(mfilt7,lib.DFoSD),
+        '11. DF/SD -> Med. filter':lib.flcompose(lib.DFoSD, mfilt7),
         }
-    gw_opts = Instance(GWOpts)
+    linescan_scope = Range(0,500,0, label='Linescan half-range')
+    linescan_width = Int(2, label="Linecan linewidth")
+    #gw_opts = Instance(GWOpts)
     dt = Float(0.2, label='sampling interval')
     fig_path = Directory("")
     ch = Enum('green', 'red', 'blue', label='Color channel')
-    glob = Str('*.tif', label='Glob', description='Image name contains...')
+    glob = Str('*_t*.tif', label='Glob', description='Image name contains...')
     leica_xml = File('', label='Leica XML', description='Leica properties file')
 
     fw_trans1 = Enum(*sorted(fw_presets.keys()),
@@ -224,54 +233,111 @@ class FrameSequenceOpts(HasTraits):
 
     export_movie_filename = File('', description='where to save the movie')
     export_fps = Float(25.0)
+    export_start = Int(0)
+    export_stop = Int(-1)
+
+
+    _export_rois_dict = File()
+    _load_rois_dict = File()
+
+    export_rois_dict_btn = Button('Export current ROIs')
+    load_rois_dict_btn = Button('Load ROIs from file')
+    _export_timeseries_file = File()
+    export_timeseries_btn = Button('Export timeseries from ROIs')
+    show_all_timeseries_btn = Button('Show all timeseries')
     
     reset_range_btn = Button("Set")
-    load_btn = Button("Load images",)
+    load_btn = Button("Load images")
     export_btn = Button('Export movie')
 
-    ## view for FSO
-    view = View(Group(Item('fig_path', ),
-                      Item('glob'),
-                      Item('leica_xml', width = 100),
-                      Item('ch'),
-                      Item('dt'),
-                      Item('load_btn', show_label=False),
-                      label='Loading'),
-                Group(Item('gw_opts', show_label=False,style='custom'),
-                      show_border=False,
-                      label='GW'),
-                Group(Item('fw_trans1'),
-                      Item('pw_trans'),
-                      Item('fw_trans2'),
-                      label='Post-process'),
-                Group(Group(Item('low_percentile'),
-                            Item('high_percentile'),
-                            Item('apply_percentile', show_label=False),
-                            label='Percentile',
-                            show_border=True,
-                            orientation='horizontal'),
-                      Group(Item('low_sd_lim'),
-                            Item('high_sd_lim'),
-                            Item('apply_sd_lim', show_label=False),
-                            label='%SD',
-                            show_border=True,
-                            orientation='horizontal'),
-                      HSplit(Item('vmax'),
-                             Item('vmin'),
-                             Item('reset_range_btn',show_label=False),
-                             show_border=True,
-                             label='Limits',
-                             springy=True),
-                      HSplit(Item('interpolation'),
-                             Item('colormap'),
-                             show_border=True,
-                             label='Matplotlib'),
-                      label='Display'),
-                Group(Item('export_movie_filename',label='Export to'),
-                      Item('export_fps', label='Frames/sec'),
-                      Item('export_btn', show_label=False),
-                      label='Export'),
-                width = 800, )
+    load_rois_dict_view = View(
+     	Item('_load_rois_dict'), 
+     	buttons = OKCancelButtons,
+     	kind = 'livemodal',
+     	width = 600,
+     	title = "Load ROIs from file",
+     	resizable = True,)
+    
+
+    export_rois_dict_view = View(
+     	Item('_export_rois_dict'), 
+     	buttons = OKCancelButtons,
+     	kind = 'livemodal',
+     	width = 600,
+     	title = "Export current ROIs to a dictionary",
+     	resizable = True,)
+
+    export_timeseries_view = View(
+     	Item('_export_timeseries_file'), 
+     	buttons = OKCancelButtons,
+     	kind = 'livemodal',
+     	width = 600,
+     	title = "Export timeseries from current ROIs",
+     	resizable = True,)
+
+    
+
+    def default_traits_view(self):
+	"""Default view for FrameSequenceOpts"""
+	## has to be a method so we can declare views for dialogs !?
+	view = View(
+	    Group(
+		Group('fig_path', 'glob',
+		      Item('leica_xml', width = 100),
+		      'ch', 'dt',
+		      Item('load_btn', show_label=False),
+		      label = 'Frame sequence',
+		      show_border=True),
+		Group(Item('linescan_width'),
+		      Item('linescan_scope'),
+		      Item('show_all_timeseries_btn',show_label=False),
+		      Item('load_rois_dict_btn',show_label=False),
+		      label = 'ROIs',
+		      show_border=True),
+		label='Open'),
+	    ## Group(Item('gw_opts', show_label=False,style='custom'),
+	    ## 	  show_border=False,
+	    ## 	  label='GW'),
+	    Group('fw_trans1', 'pw_trans', 'fw_trans2',
+		  label='Post-process'),
+	    Group(Group(Item('low_percentile'),
+			Item('high_percentile'),
+			Item('apply_percentile', show_label=False),
+			label='Percentile',
+			show_border=True,
+			orientation='horizontal'),
+		  Group(Item('low_sd_lim'),
+			Item('high_sd_lim'),
+			Item('apply_sd_lim', show_label=False),
+			label='%SD',
+			show_border=True,
+			orientation='horizontal'),
+		  HSplit(Item('vmax'),
+			 Item('vmin'),
+			 Item('reset_range_btn',show_label=False),
+			 show_border=True,
+			 label='Limits',
+			 springy=True),
+		  HSplit(Item('interpolation'),
+			 Item('colormap'),
+			 show_border=True,
+			 label='Matplotlib'),
+		  label='Display'),
+	    Group(Group(Item('export_movie_filename',label='Export to'),
+			Item('export_fps', label='Frames/sec'),
+			Item('export_start', label='Start frame'),
+			Item('export_stop', label='Stop frame'),
+			Item('export_btn', show_label=False),
+			show_border=True,
+			label='Movie'),
+		  Group('export_rois_dict_btn',
+			'export_timeseries_btn',
+			show_border=True,
+			show_labels=False,
+			label='ROIs'),
+		  label="Export"),
+	    width = 800, )
+	return view
     def __init__(self, parent):
         self.parent = parent
         self.get_fs = self.get_fs2
@@ -279,7 +345,7 @@ class FrameSequenceOpts(HasTraits):
     def _fig_path_changed(self):
         png_pattern = str(self.fig_path + os.sep + '*.png')
         if len(fseq.sorted_file_names(png_pattern)) > 30:
-            self.glob = '*.png'
+            self.glob = '*_t*.png'
         self.leica_xml = leica.get_xmljob(self.fig_path)
 
     def _glob_changed(self):
@@ -357,6 +423,11 @@ class FrameSequenceOpts(HasTraits):
             self.fs2_needs_reload = False
         return self.fs2
 
+    def _show_all_timeseries_btn_fired(self):
+	print 'in show_all_timeseries_btn_fired'
+	self.parent.picker.show_timeseries()
+	pass
+
     def _reset_range_btn_fired(self):
         self.vmin, self.vmax = self.fs2.data_range()
 
@@ -376,31 +447,63 @@ class FrameSequenceOpts(HasTraits):
         print "fns1 set"
         self.fs2_needs_reload = True
         self.get_fs2()
+	print 'fs2 set'
         return
 
-    def _gw_opts_default(self):
-        gw_opts = GWOpts(self)
-        return gw_opts
+    ## def _gw_opts_default(self):
+    ##     gw_opts = GWOpts(self)
+    ##     return gw_opts
+
 
     def _export_btn_fired(self):
         seq = self.get_fs2()
         seq.export_mpeg(self.export_movie_filename,
                         fps = self.export_fps,
+			start = self.export_start,
+			stop = self.export_stop,
                         cmap=self.colormap,
                         interpolation = self.interpolation,
                         vmin = self.vmin,
                         vmax=self.vmax)
 
+    def _load_rois_dict_btn_fired(self):
+	ui = self.edit_traits(view='load_rois_dict_view')
+	if ui.result == True:
+	    print self._load_rois_dict
+	    picker = self.parent.picker
+	    picker.load_rois(self._load_rois_dict)
+
+
+    def _export_rois_dict_btn_fired(self):
+	print "in _export_rois_dict_btn_fired"
+	ui = self.edit_traits(view='export_rois_dict_view')
+	if ui.result == True:
+	    print self._export_rois_dict
+	    picker = self.parent.picker
+	    picker.save_rois(self._export_rois_dict)
+
+    def _export_timeseries_btn_fired(self):
+	print "_export_timeseries_btn_fired"
+	ui = self.edit_traits(view='export_timeseries_view')
+	if ui.result == True:
+	    print self._export_timeseries_file
+	    picker = self.parent.picker
+	    picker.save_time_series_to_file(self._export_timeseries_file)
+	
     def _load_btn_fired(self):
         if self.parent.frames is not None:
             del self.parent.frames
             self.parent.frames = None
         self.reset_fs()
+	print 'reset_fs done'
         self.vmin, self.vmax = self.fs2.data_range()
+	print 'vmin,vmax done'
         self.dt = self.fs.dt
+	print 'dt done'
         self.parent._recalc_btn_fired()
+	print 'recalc'
 
-class Test(HasTraits):
+class FrameViewer(HasTraits):
     fso = Instance(FrameSequenceOpts)
     figure = Instance(Figure, ())
     max_frames = Int(100)
@@ -410,8 +513,8 @@ class Test(HasTraits):
     time_stat = String()
     recalc_btn = Button("Recalc frames")
 
-    frame_fwd_btn = Button('frame +')
-    frame_back_btn = Button('- frame')
+    frame_fwd_btn = Button('>')
+    frame_back_btn = Button('<')
 
 
     view = View(HSplit(Group(Item('recalc_btn', show_label=False),
@@ -429,9 +532,6 @@ class Test(HasTraits):
                        Item('fso', style='custom'),
                        springy = True,
                        show_labels=False),
-                #menubar = MenuBar(Separator(),
-                #                  CloseAction,
-                #                  name = 'File'),
                 width=1200,
                 height=600,
                 resizable=True,
@@ -495,8 +595,9 @@ class Test(HasTraits):
         if hasattr(self, 'picker'):
             self.picker.disconnect()
         self.picker = ifui.Picker(fs2)
+	self.picker.caller = self
         self.axes.cla()
-        _,self.pl = self.picker.start(ax=self.axes, legend_type='axlegend',
+        _,self.pl,_ = self.picker.start(ax=self.axes, legend_type='axlegend',
                                       cmap = self.fso.colormap,
                                       interpolation = self.fso.interpolation)
         self.pl.axes.figure.canvas.draw()
@@ -507,7 +608,7 @@ class Test(HasTraits):
 
 
 def main():
-    Test().configure_traits()
+    FrameViewer().configure_traits()
 
 if __name__ == "__main__":
     main()
