@@ -19,6 +19,7 @@ ifnot = lib.ifnot
 in_circle = lib.in_circle
 
 from scipy import signal
+from scipy.interpolate import splrep, splev, splprep
 
 
 
@@ -38,7 +39,7 @@ def line_from_struct(line_props):
     lp = line_props.copy()
     _ = lp.pop('func')
     xdata, ydata = lp.pop('xdata'), lp.pop('ydata')
-    return Line2D(xdata, ydata,**lp)
+    return LineScan(xdata, ydata,**lp)
 
 
 import pickle
@@ -47,14 +48,14 @@ import random
 vowels = "aeiouy"
 consonants = "qwrtpsdfghjklzxcvbnm"
 
-
 def rand_tag():
     return ''.join(map(random.choice,
                        (consonants, vowels, consonants)))
 def tags_iter(tag_id =1):
     while True:
-        yield 'roi%03d'%tag_id
+        yield 'r%02d'%tag_id
         tag_id +=1
+        
 def rand_tags_iter():
     while True: yield rand_tag()
 
@@ -68,29 +69,34 @@ def unique_tag(tags, max_tries = 1e4, tagger = tags_iter()):
     return "Err"
 
 
-def nearest_item_ind(items, xy, fn = lambda a: a):
-    """
-    Index of the nearest item from a collection.
-    Arguments: collection, position, selector
-    """
-    return lib.min1(lambda p: lib.eu_dist(fn(p), xy), items)
-
-
-
+## def nearest_item_ind(items, xy, fn = lambda a: a):
+##     """
+##     Index of the nearest item from a collection.
+##     Arguments: collection, position, selector
+##     """
+##     return lib.min1(lambda p: lib.eu_dist(fn(p), xy), items)
 
 def line_from_points(p1,p2):
     p1 = map(float, p1)
     p2 = map(float, p2)
     k = (p1[1] - p2[1])/(p1[0] - p2[0])
     b = p1[1] - p1[0]*k
-    return lambda x: k*x + b
+    return k, b
 
+def translate(p1,p2):
+    """returns translation vector used to
+    ||-translate line segment p1,p2 in an orthogonal direction"""
+    L = lib.eu_dist(p1, p2)
+    v = [np.array(p, ndmin=2).T for p in p1,p2]
+    T = np.array([[0, -1], [1, 0]]) # basic || translation vector
+    tv = np.dot(T, v[0] - v[1])/float(L)
+    return tv
 
-def extract_line(data, xind, f):
+def line_reslice1(data, xind, f):
     return np.array([data[int(i),int(f(i))] for i in xind])
 
 
-def extract_line2(data, p1, p2):
+def line_reslice2(data, p1, p2):
     L = int(lib.eu_dist(p1,p2))
     f = lambda x1,x2: lambda i: int(x1 + i*(x2-x1)/L)
     return np.array([data[f(p1[1],p2[1])(i), f(p1[0],p2[0])(i)]
@@ -109,28 +115,28 @@ def rezip(a):
     return zip(*a)
 
 
-def view_fseq_frames(fseq, vmin = None, vmax = None):
+def view_fseq_frames(fseq, vmin = None, vmax = None,cmap='gray'):
     f = pl.figure()
     axf = pl.axes()
     frame_index = [0]
-    frames = fseq.as3darray()
-    Nf = len(frames)
+    Nf = fseq.length()
 
     if vmax is None:
-        vmax = np.max(map(np.max, frames))
+        vmax = fseq.data_range()[1]
 
     if vmin is None:
-        vmin = np.min(map(np.min, frames))
+        vmin = fseq.data_range()[0]
 
-
-    sy,sx = fseq.shape()
+    sh = fseq.shape()
+    sy,sx = sh[:2]
+	
     dy,dx, scale_setp = fseq.get_scale()
 
-    plf = axf.imshow(frames[0],
+    plf = axf.imshow(fseq[0],
                      extent = (0, sx*dx, 0, sy*dy),
                      interpolation = 'nearest',
                      vmax = vmax, vmin = vmin,
-                     aspect = 'equal', cmap=mpl.cm.gray)
+                     aspect = 'equal', cmap=cmap)
     if scale_setp:
         pl.ylabel('um')
         pl.xlabel('um')
@@ -138,21 +144,177 @@ def view_fseq_frames(fseq, vmin = None, vmax = None):
     def skip(event,n=1):
         fi = frame_index[0]
         key = hasattr(event, 'button') and event.button or event.key
-        if key in (4,'4','down','left'):
+        if key in (4,'4','down','left','p'):
             fi -= n
-        elif key in (5,'5','up','right'):
+        elif key in (5,'5','up','right','n'):
             fi += n
         fi = fi%Nf
-        plf.set_data(frames[fi])
+        plf.set_data(fseq[fi])
         axf.set_title('%03d (%3.3f sec)'%(fi, fi*fseq.dt))
         frame_index[0] = fi
         f.canvas.draw()
     f.canvas.mpl_connect('scroll_event',skip)
-    #f.canvas.mpl_connect('key_press_event', skip)
-    f.canvas.mpl_connect('key_press_event',lambda e: skip(e,10))
+    f.canvas.mpl_connect('key_press_event',skip)
+
+
+def resample_velocity(rad, v,rstart=10.0):
+    rstop = rad[-1]
+    xnew = np.arange(rstart, rstop, 0.25)
+    return xnew, np.interp(xnew,rad,v)
+
+
+DM_help_msg =  """
+Diameter Manager:   
+- left-click or type 'i' to add point under mouse
+- rigth-click or type 'd' to remove point under mouse
+- type 'a' to calculate propagation velocity
+"""
+
+
+class GWExpansionMeasurement1:
+    def __init__(self, ax):
+	self.ax = ax
+	self.points = []
+	self.line = None # user-picked points
+	self.line2 = None
+	self.line_par = None
+	self._ind = None
+	self.epsilon = 5
+	self.canvas = ax.figure.canvas
+	self.center = None
+	self.velocities = None
+	self.smooth = 1
+	#print self.ax, self.ax.figure, self.canvas
+	cf = self.canvas.mpl_connect
+	self.cid = {
+            'press': cf('button_press_event', self._on_button_press),
+            #'release': cf('button_release_event', self.on_release),
+            #'motion': cf('motion_notify_event', self.on_motion),
+            #'scroll': cf('scroll_event', self.on_scroll),
+            'type': cf('key_press_event', self._on_key_press)
+            }
+	pl.show()
+	print DM_help_msg
+	
+    def get_ind_closest(self,event):
+	xy = np.asarray(self.points)
+	if self.line is not None:
+	    xyt = self.line.get_transform().transform(xy)
+	    xt, yt = xyt[:,0], xyt[:,1]
+	    d = ((xt-event.x)**2 + (yt-event.y)**2)**0.5
+	    indseq = np.nonzero(np.equal(d, np.amin(d)))[0]
+	    ind = indseq[0]
+	    if d[ind]>=self.epsilon:
+		ind = None
+	else: ind = None
+        return ind
+    def add_point(self, event):
+	p = event.xdata, event.ydata
+	ind = self.get_ind_closest(event)
+	if ind is not None:
+	    print "too close to an existing point"
+	    return
+	self.points.append(p)
+	self.points.sort(key = lambda u:u[0])
+	xd, yd = rezip(self.points)
+	if self.line is None:
+	    self.line = pl.Line2D(xd,yd,marker='o',
+				  ls = '--',
+				  color='r', alpha=0.75,
+				  markerfacecolor='r')
+				  #animated='True')
+	    self.ax.add_line(self.line)
+	    #print 'added line'
+	else:
+	    self.line.set_data([xd,yd])
+    def remove_point(self, event):
+	ind = self.get_ind_closest(event)
+	if ind is not None:
+	    self.points = [pj for j,pj in enumerate(self.points) if j !=ind]
+	    self.line.set_data(rezip(self.points))
+    def action(self,min_r = 5.):
+	xd, yd = map(np.array, rezip(self.points))
+	par = np.polyfit(xd,yd,2)
+	xfit_par = np.linspace(xd[0], xd[-1], 256)
+	yfit_par = np.polyval(par, xfit_par)
+	v = np.gradient(np.asarray(yd))
+	dx = np.gradient(np.asarray(xd))
+	tck,u = splprep([xd,yd],s=self.smooth)	
+	unew = np.linspace(0,1.,100)
+	out = splev(unew,tck)
+	if self.line_par:
+	    self.line_par.set_data(xfit_par, yfit_par)
+	else:
+	    self.line_par = pl.Line2D(xfit_par, yfit_par, color='cyan')
+	    self.ax.add_line(self.line_par)
+	if self.line2:
+	    self.line2.set_data(out[0], out[1])
+	else:
+	    self.line2 = pl.Line2D(out[0], out[1], color='w',
+				   lw=2,alpha=0.75)
+	    self.ax.add_line(self.line2)
+	x,y = out[0], out[1]
+	midpoint = np.argmin(y)
+	lh_r = abs(x[:midpoint]-x[midpoint]) #left branch
+	rh_r = x[midpoint:]-x[midpoint] # right branch
+	vel = lambda d,t: np.abs(np.gradient(d)/np.gradient(t))
+	rh_v = vel(rh_r[rh_r>=min_r],y[midpoint:][rh_r>=5])
+	lh_v = vel(lh_r[lh_r>=min_r],y[:midpoint][lh_r>=5])
+	rh_r = rh_r[rh_r>=min_r]
+	lh_r = lh_r[lh_r>=min_r]
+	v_at_r = lambda rv,vv,r0: vv[np.argmin(np.abs(rv-r0))]
+	vmean_at_r = lambda r0:\
+		     np.mean([v_at_r(lh_r,lh_v,r0),
+			      v_at_r(rh_r,rh_v,r0)])
+	
+	ax = pl.figure().add_subplot(111);
+	ax.plot(lh_r[lh_r>min_r],lh_v,'b-',lw=2)
+	ax.plot(rh_r[rh_r>min_r],rh_v,'g-',lw=2)
+	ax.legend(['left-hand branch','right-hand branch'])
+	ax.set_xlabel('radius, um'); ax.set_ylabel('velocity, um/s')
+	ax.set_title('average velocity at 15 um: %02.2f um/s'%\
+		 vmean_at_r(15.))
+	ax.grid(True)
+	self.canvas.draw()
+	print "-------- Velocities ----------"
+	print np.array([(rx, vmean_at_r(rx)) for rx in range(8,22,2)])
+	print "------------------- ----------"
+	self.velocities = [[lh_r[::-1], lh_v[::-1]],
+			   [rh_r, rh_v]]
+	return 
+    def _on_button_press(self,event):
+	if not self.event_ok(event): return
+	x,y = event.xdata, event.ydata
+	if event.button == 1:
+	    self.add_point(event)
+	## elif event.button == 2:
+	##     if self.center is None:
+	## 	self.center = pl.Line2D([x],[y],marker='*',color='m',
+	## 				markersize=12)
+	## 	self.ax.add_line(self.center)
+	##     else:
+	## 	self.center.set_data([[x],[y]])
+	elif event.button == 3:
+	    self.remove_point(event)
+	self.canvas.draw()
+	    
+    def _on_key_press(self, event):
+	if not self.event_ok(event): return
+	if event.key == 'i':
+	    self.add_point(event)
+	elif event.key == 'd':
+	    self.remove_point(event)
+	elif event.key == 'a':
+	    self.action()
+	self.canvas.draw()
+    def event_ok(self, event):
+        return event.inaxes == self.ax and \
+               pl.get_current_fig_manager().toolbar.mode ==''
+
 
 
 class DraggableObj:
+    """Basic class for objects that can be dragged on the screen"""
     verbose = True
     def __init__(self, obj, parent,):
         self.obj = obj
@@ -202,11 +364,8 @@ class DraggableObj:
         pass
 
     def on_scroll(self, event): pass
-
     def on_type(self, event): pass
-    
-    def on_press(self, event):
-        pass
+    def on_press(self, event):   pass
 
     def disconnect(self):
         map(self.obj.axes.figure.canvas.mpl_disconnect,
@@ -273,33 +432,90 @@ class LineScan(DraggableObj):
             y0 = y[1] - y[0]
             self.pressed = event.xdata, event.ydata
         elif event.button == 2:
-            self.show_timeview()
+            self.dm = self.show_timeview()
+
+    def on_type(self, event):
+	if not self.event_ok(event, True):
+	    return
+	if event.key == '/':
+	    self.diameter_manager = self.show_timeview()
 
     def transform_point(self, p):
         return p[0]/self.dx, p[1]/self.dy
 
-    def get_timeview(self):
+    def get_timeview(self,hwidth=2,frange=None):
         points = map(self.transform_point, self.check_endpoints())
-        timeview = np.array([extract_line2(frame, *points) for frame in
-                             self.parent.fseq.frames()])
-        return timeview,points
+	if frange is None:
+	    if hasattr(self.parent, 'caller'): # is called from frame_viewer?
+		caller = self.parent.caller
+		hwidth = caller.fso.linescan_width # overrides argument
+		half_scope = caller.fso.linescan_scope
+		if half_scope == 0:
+		    frames = lambda : self.parent.fseq.frames()
+		else:
+		    fi = caller.frame_index
+		    fstart = max(0,fi-half_scope)
+		    fstop = min(self.parent.fseq.length(),fi+half_scope)
+		    frange = slice(fstart,fstop)
+		    frames = lambda : self.parent.fseq[frange]
+	    else:
+		frames = lambda : self.parent.fseq.frames()
+	else:
+	    frames = lambda : self.parent.fseq[frange]
+	tv = np.reshape(translate(*points),-1) # translation vector
+	timeview = lambda pts: np.array([line_reslice2(frame, *pts)
+					 for frame in frames()])
+	if hwidth > 0:
+	    plist = [points]
+	    plist += [points + s*k*tv
+		      for k in range(1, hwidth+1) for s in -1,1]
+	    out = np.mean(map(timeview, plist), axis=0)
+	else:
+	    out = timeview(points)
+        return out,points
 
-    def show_timeview(self):
-        timeview,points = self.get_timeview()
+    def get_full_projection(self, fn = np.mean,axis=1,
+			    mode='constant',
+			    output = 'result'):
+	"""
+	if "output" is 'function' return a function to rotate and project any  data
+	if "output is 'result', return a projection of rotated data, associated
+	with the frame sequence of the Picker.
+	"""
+	from scipy.ndimage.interpolation import rotate
+	points = map(self.transform_point, self.check_endpoints())
+	k, b = line_from_points(*points)
+	phi = np.rad2deg(np.arctan(k))
+	def _(data):
+	    rot = rotate(data, phi, (1,2), mode=mode)
+	    rot = np.ma.masked_less_equal(rot, 0)
+	    return np.array(fn(rot, axis=axis))
+	if output=='result':
+	    return _(self.parent.fseq.as3darray())
+	elif output == 'function':
+	    return _
+	
+
+
+    def show_timeview(self,frange=None,hwidth=2):
+        timeview,points = self.get_timeview(frange=frange,hwidth=hwidth)
         if timeview is not None:
             fseq = self.parent.fseq
-            ax = pl.figure().add_subplot(111)
+	    f = pl.figure()
+            ax = f.add_subplot(111)
             # TODO: work out if sx differs from sy
             ax.imshow(timeview,
                       extent=(0, self.dx*timeview.shape[1], 0,
-                              fseq.dt*self.parent.length()),
+                              fseq.dt*timeview.shape[0]),
                       )
                       #aspect='equal')
             if self.scale_setp:
                 ax.set_xlabel('um')
             ax.set_ylabel('time, sec')
             ax.set_title('Timeview for '+ self.tag)
-            ax.figure.show()
+	    pc = GWExpansionMeasurement1(ax)
+	    return pc
+	    
 
     def to_struct(self):
         l = self.obj
@@ -312,7 +528,7 @@ class LineScan(DraggableObj):
 
 
 class CircleROI(DraggableObj):
-    "Draggable Circle ROI"
+    """Draggable Circle ROI"""
     step = 1
     def on_scroll(self, event):
         if not self.event_ok(event, True): return
@@ -332,7 +548,9 @@ class CircleROI(DraggableObj):
         tags = [self.tag]
         if event.key in ['t', '1']:
             self.parent.show_timeseries(tags)
-        if event.key in ['T', '!']:
+        elif event.key in ['c']:
+            self.parent.show_xcorrmap(self.tag)
+        elif event.key in ['T', '!']:
             self.parent.show_timeseries(tags, normp=True)
         elif event.key in ['w', '2']:
             self.parent.show_spectrogram_with_ts(self.tag)
@@ -382,17 +600,32 @@ class CircleROI(DraggableObj):
         else:
             self.tagtext.set_position((x,y))
 
-
-    def get_timeview(self, normp=False):
+    def in_circle(self, shape):
         roi = self.obj
         c = roi.center[0]/self.dx, roi.center[1]/self.dy
-        fn = in_circle(c, roi.radius)
-        shape = self.parent.fseq.shape()
+        fn = lib.in_circle(c, roi.radius/self.dx)
         X,Y = np.meshgrid(*map(range, shape[::-1]))
-        v = self.parent.fseq.mask_reduce(fn(X,Y))
+        return fn(X,Y)
+
+    def get_timeview(self, normp=False):
+	"""return timeseries from roi
+
+	Parameters:
+	  - if normp is False, returns raw timeseries v
+	  - if normp is a function f, returns f(v)
+	  - if normp is a number N, returns :math:`\\Delta v/v_0`, where
+            :math:`v_0` is calculated over N first points 
+	  - else, returns :math:`\\Delta v/\\bar v` 
+
+	"""
+        shape = self.parent.fseq.shape()
+        v = self.parent.fseq.mask_reduce(self.in_circle(shape))
         if normp:
-            Lnorm = type(normp) is int and normp or len(v)
-            return (v-np.mean(v[:Lnorm]))/np.std(v[:Lnorm])
+	    if callable(normp):
+		return normp(v)
+	    else:
+		Lnorm = type(normp) is int and normp or len(v)
+		return lib.DFoF(v)
         else: return v
 
     def to_struct(self):
@@ -460,8 +693,8 @@ class RectFollower(DragRect):
         acc = {}
         sw, sh = self.search_width, self.search_height
         _,limh, limw = self.arr.shape
-        for w in range(max(0,-sw/2), min(sw/2,limw)):
-            for h in range(max(0,-sh/2), min(limh,sh/2)):
+        for w in np.arange(max(0,-sw/2), min(sw/2,limw)):
+            for h in np.arange(max(0,-sh/2), min(limh,sh/2)):
                 s = self.toslice(h,w)
                 d = measure(frame[s], template)
                 acc[(w,h)] = d
@@ -534,6 +767,7 @@ def track_vessels(frames, width=30, height=60, measure = sse_measure):
 class Picker:
     verbose = True
     def __init__(self, fseq):
+        self._corrfn = 'pearson'
         self.cw = color_walker()
 
         self._show_legend=False
@@ -542,6 +776,7 @@ class Picker:
         self._Nf = None
         self.roi_objs = {}
         self.min_length = 5
+	self.frame_index = 0 
 
         pass
 
@@ -557,7 +792,7 @@ class Picker:
         x,y = round(event.xdata), round(event.ydata)
         if event.button is 1 and \
            not self.any_roi_contains(event):
-            label = unique_tag(self.roi_tags(), self.tagger)
+            label = unique_tag(self.roi_tags(), tagger=self.tagger)
             c = pl.Circle((x,y), 5, alpha = 0.5,
                        label = label,
                        color=self.cw.next())
@@ -567,7 +802,7 @@ class Picker:
             #drc.connect()
         self.legend()    
         #self.ax1.legend()
-        pl.draw()
+        self.ax1.figure.canvas.draw()
 
     def legend(self):
         if self._show_legend == True:
@@ -583,11 +818,13 @@ class Picker:
                     self.redraw()
             except Exception as e:
                     print "Picker: can't make legend because ", e
+    def event_canvas_ok(self, event):
+	pred = event.inaxes !=self.ax1 or \
+		   self.any_roi_contains(event) or \
+		   pl.get_current_fig_manager().toolbar.mode !=''
+	return not pred
     def on_press(self, event):
-        if event.inaxes !=self.ax1 or \
-               self.any_roi_contains(event) or \
-               event.button != 3 or \
-               pl.get_current_fig_manager().toolbar.mode !='':
+	if not self.event_canvas_ok(event) or event.button != 3 :
             return
         self.pressed = event.xdata, event.ydata
         axrange = self.ax1.get_xbound() + self.ax1.get_ybound()
@@ -613,7 +850,7 @@ class Picker:
         if self.any_roi_contains(event): return
         if not hasattr(self, 'curr_line_handle'): return
         if len(self.curr_line_handle.get_xdata()) > 1:
-            tag = unique_tag(self.roi_tags(), self.tagger)
+            tag = unique_tag(self.roi_tags(), tagger=self.tagger)
             self.curr_line_handle.set_label(tag)
             newline = LineScan(self.curr_line_handle, self)
             if newline.length() > self.min_length:
@@ -633,14 +870,6 @@ class Picker:
         return
 
 
-    ## def any_line_contains(self,event):
-    ##     "Checks if event is contained by any ROI"
-    ##     if len(self.roi_objs) < 1 : return False
-    ##     return reduce(lambda x,y: x or y,
-    ##                   [roi.obj.contains(event)[0]
-    ##                    for roi in self.roi_objs.values()])
-
-
     def any_roi_contains(self,event):
         "Checks if event is contained by any ROI"
         if len(self.roi_objs) < 1 : return False
@@ -666,8 +895,8 @@ class Picker:
         "Load stored ROIs from a file"
         saved_rois = pickle.load(file(fname))
         rois = [x['func'](x) for x in saved_rois]
-        circles = filter(lambda x: isinstance(x, Circle), rois)
-        lines = filter(lambda x: isinstance(x, Line2D), rois)
+        circles = filter(lambda x: isinstance(x, pl.Circle), rois)
+        lines = filter(lambda x: isinstance(x, LineScan), rois)
         map(self.ax1.add_patch, circles) # add points to the axes
         map(self.ax1.add_line, lines) # add points to the axes
         self.roi_objs.update(dict([(c.get_label(), CircleROI(c,self))
@@ -676,26 +905,43 @@ class Picker:
                                    for l in lines]))
 
         #self.ax1.legend()
-        pl.draw() # redraw the axes
+        self.ax1.figure.canvas.draw() # redraw the axes
         return
 
     def start(self, roi_objs={}, ax=None, legend_type = 'figlegend',
+              mean_frame =True,
+              vmax = None, vmin = None, 
               cmap = 'gray',
               interpolation = 'nearest'):
         "Start picking up ROIs"
         self.tagger = tags_iter()
         self.drcs = {}
-        self.ax1 = ifnot(ax, pl.figure().add_subplot(111))
-        self.fig = self.ax1.figure
+	if ax is None:
+	    self.fig = pl.figure()
+	    self.ax1 = self.fig.add_subplot(111)
+	else:
+	    self.ax1 = ax
+	    self.fig = self.ax1.figure
+        #self.ax1 = ifnot(ax, pl.figure().add_subplot(111))
+        #self.fig = self.ax1.figure
         self.legtype = legend_type
         self.pressed = None
 
         dx,dy, scale_setp = self.fseq.get_scale()
-        sy,sx = self.fseq.shape()
-        vmin,vmax = self.fseq.data_range()
+	sy,sx = self.fseq.shape()[:2]
+        avmin,avmax = self.fseq.data_range()
+        if vmin is None: vmin = avmin
+        if vmax is None: vmax = avmax
         if hasattr(self.fseq, 'ch'):
-            pl.title("Channel: %s" % ('red', 'green')[self.fseq.ch] )
-        self.pl = self.ax1.imshow(self.fseq.mean_frame(),
+            pl.title("Channel: %s" % ('red', 'green','blue')[self.fseq.ch] )
+        if type(mean_frame) is np.ndarray:
+	    f = mean_frame
+	elif mean_frame:
+            f = self.fseq.mean_frame()
+        else:
+            f = self.fseq.frames().next()
+	self.mean_frame = f
+        self.pl = self.ax1.imshow(f,
                                   extent = (0, sx*dx, 0, sy*dy),
                                   interpolation = interpolation,
                                   aspect='equal',
@@ -707,8 +953,34 @@ class Picker:
 
         self.disconnect()
         self.connect()
-
-        return self.ax1, self.pl
+	#self.fig.show()
+        return self.ax1, self.pl, self
+    def frame_skip(self,event, n=1):
+	if not self.event_canvas_ok(event):
+	    return
+	Nf = self.fseq.length()
+	fi = self.frame_index
+	key = hasattr(event, 'button') and event.button or event.key
+	prev_keys = [4,'4','down','left','p']
+	next_keys = [5,'5','up','right','n']
+	home_keys =  ['h','q','z']
+	known_keys = prev_keys+ next_keys+home_keys
+	if key in known_keys:
+	    if key in home_keys:
+		show_f = self.mean_frame
+		_title = ""
+	    else:
+		if key in prev_keys:
+		    fi -= n
+		elif key in next_keys:
+		    fi += n
+		fi = fi%Nf
+		self.frame_index = fi
+		_title = '%03d (%3.3f sec)'%(fi, fi*self.fseq.dt)
+		show_f = self.fseq[fi]
+	    self.pl.set_data(show_f)
+	    self.ax1.set_title(_title)
+	    self.fig.canvas.draw()
 
     def connect(self):
         "connect all the needed events"
@@ -719,23 +991,22 @@ class Picker:
             'press': cf('button_press_event', self.on_press),
             'release': cf('button_release_event', self.on_release),
             'motion': cf('motion_notify_event', self.on_motion),
+	    'scroll':cf('scroll_event',self.frame_skip),
+	    'type':cf('key_press_event',self.frame_skip),	    
             }
     def disconnect(self):
         if hasattr(self, 'cid'):
             print "disconnecting old callbacks"
             map(self.fig.canvas.mpl_disconnect, self.cid.values())
             
-    def list_roi_timeseries_from_labels(self, roi_tags, **keywords):
-        "Returns timeseres for a list of roi labels"
-        return [self.roi_timeseries_from_label(tag, **keywords)
-                for tag in roi_tags]
-
     def isCircleROI(self,tag):
         return isinstance(self.roi_objs[tag], CircleROI)
-    
+
+    def get_circle_roi_tags(self):
+	return sorted(filter(self.isCircleROI, self.roi_objs.keys()))
     def get_timeseries(self, rois=None, normp=False):
         rois = ifnot(rois,
-                     filter(self.isCircleROI, self.roi_objs.keys()))
+                     sorted(filter(self.isCircleROI, self.roi_objs.keys())))
         return [self.roi_objs[tag].get_timeview(normp)
                 for tag in  rois]
 
@@ -744,7 +1015,7 @@ class Picker:
         return np.arange(0,Nf*dt, dt)[:Nf]
 
     def save_time_series_to_file(self, fname, normp = False):
-        rois = filter(self.isCircleROI, self.roi_objs.keys())
+        rois = sorted(filter(self.isCircleROI, self.roi_objs.keys()))
         ts = self.get_timeseries(normp=normp)
         t = self.timevec()        
         fd = file(fname, 'w')
@@ -762,10 +1033,18 @@ class Picker:
 
 
     def show_timeseries(self, rois = None, **keywords):
+	#print 'in Picker.show_timeseries()'
         t = self.timevec()
         for x,tag,roi,ax in self.roi_show_iterator(rois, **keywords):
-            ax.plot(t, x, color = roi.get_facecolor())
-            pl.xlim((0,t[-1]))
+	    if self.isCircleROI(tag):
+		ax.plot(t, x, color = roi.get_facecolor())
+	    else:
+		sh = x.shape
+		ax.imshow(x.T, extent=(t[0],t[-1],0,sh[1]*self.fseq.dx),
+			  aspect='auto',cmap=lib.swanrgb)
+            pl.xlim(0,t[-1])
+	ax.figure.show()
+	    
     def show_ffts(self, rois = None, **keywords):
         L = self.length()
         freqs = np.fft.fftfreq(int(L), self.dt)[1:L/2]
@@ -773,13 +1052,22 @@ class Picker:
             y = abs(np.fft.fft(x))[1:L/2]
             ax.plot(freqs, y**2)
         ax.set_xlabel("Frequency, Hz")
+
     def show_xcorrmap(self, roitag, figsize=(6,6),
                       **kwargs):
-        signal = self.get_timeseries([roitag],normp=True)[0]
-        xcmap = fnmap.xcorrmap(self.fseq, signal, **kwargs)
-        pl.figure(figsize=figsize)
-        pl.imshow(xcmap, aspect = 'equal')
-        pl.title('Correlation to %s'%roitag)
+        from scipy import ndimage
+        roi =  self.roi_objs[roitag]
+        signal = self.get_timeseries([roitag],normp=False)[0]
+        xcmap = fnmap.xcorrmap(self.fseq, signal, corrfn=self._corrfn,**kwargs)
+        mask = roi.in_circle(xcmap.shape)
+        xshow = np.ma.masked_where(mask,xcmap)
+        fig = pl.figure(figsize=figsize)
+        ax = fig.add_subplot(111)
+        vmin, vmax = xshow.min(), xshow.max()
+        im = ax.imshow(ndimage.median_filter(xcmap,3), aspect = 'equal', vmin=vmin,
+                       vmax=vmax,cmap=lib.swanrgb)
+        pl.colorbar(im, ax=ax)
+        ax.set_title('Correlation to %s'%roitag)
         return xcmap
 
     def show_spectrograms(self, rois = None, freqs = None,
@@ -810,7 +1098,7 @@ class Picker:
         if not self.isCircleROI(roitag):
             print "This is not a circle ROI, exiting"
             return
-        signal = self.get_timeseries([roitag],normp=normp)[0]
+        signal = self.get_timeseries([roitag],normp=lib.DFoSD)[0]
         Ns = len(signal)
         f_s = 1/self.dt
         freqs = ifnot(freqs,self.default_freqs())
@@ -849,21 +1137,31 @@ class Picker:
                            0.5/self.dt, num=nfreqs)
 
     def roi_show_iterator(self, rois = None,
-                              normp=False):
-            rois = ifnot(rois,
-                         filter(self.isCircleROI, self.roi_objs.keys()))
-            L = len(rois)
-            if L < 1: return
-            fig = pl.figure(figsize=(8,4.5), dpi=80)
-            for i, roi_label in enumerate(rois):
-                roi = self.roi_objs[roi_label].obj
-                ax = fig.add_subplot(L,1,i+1)
-                x = self.roi_objs[roi_label].get_timeview(normp=normp)
-                if i == L-1:
-                    ax.set_xlabel("time, sec")
-                ax.set_ylabel(roi_label)
-                yield x, roi_label, roi, ax
-            fig.show()
+                              **kwargs):
+	#rois = ifnot(rois,
+        #             sorted(filter(self.isCircleROI, self.roi_objs.keys())))
+	#print 'in Picker.roi_show_iterator'
+	rois = ifnot(rois, sorted(self.roi_objs.keys()))
+	
+	L = len(rois)
+	if L < 1: return
+	fig = pl.figure(figsize=(8,4.5), dpi=80)
+	for i, roi_label in enumerate(rois):
+	    roi = self.roi_objs[roi_label]
+	    if i == 0:
+		ax = fig.add_subplot(L,1,i+1)
+		ax0 = ax
+	    else:
+		ax = fig.add_subplot(L,1,i+1, sharex = ax0)
+	    if self.isCircleROI(roi_label):
+		x = roi.get_timeview(**kwargs)
+	    else:
+		x,points = roi.get_timeview(**kwargs)
+	    if i == L-1:
+		ax.set_xlabel("time, sec")
+	    ax.set_ylabel(roi_label)
+	    yield x, roi_label, roi.obj, ax
+
 
 
     def show_xwt_roi(self, tag1, tag2, freqs=None,
@@ -885,20 +1183,42 @@ class Picker:
         t = self.timevec()
 
         pl.figure();
-        ax1= subplot(211);
+        ax1= pl.subplot(211);
         roi1,roi2 = self.roi_objs[tag1], self.roi_objs[tag2]
-        plot(t,s1,color=roi1.get_color(), label=tag1)
-        plot(t,s2,color=roi2.get_color(), label = tag2)
+        pl.plot(t,s1,color=roi1.get_color(), label=tag1)
+        pl.plot(t,s2,color=roi2.get_color(), label = tag2)
         #legend()
-        ax2 = subplot(212, sharex = ax1);
+        ax2 = pl.subplot(212, sharex = ax1);
         ext = (t[0], t[-1], freqs[0], freqs[-1])
-        ax2.imshow(res, extent = ext, cmap = lib.swanrgb())
+        ax2.imshow(res, extent = ext, cmap = lib.swanrgb)
         #self.cone_infl(freqs,wavelet)
         #self.confidence_contour(res,2.0)
 
+    def show_roi_xcorrs(self, corrfn = None, normp = lib.DFoSD,
+			getter = lambda x: x[0]):
+	if corrfn == None:
+	    corrfn = fnmap.stats.pearsonr
+	rois = sorted(filter(self.isCircleROI, self.roi_objs.keys()))
+	ts = self.get_timeseries(normp=normp)
+        t = self.timevec()
+	nrois = len(rois)
+	if nrois < 2:
+	    print "not enough rois, pick more"
+	    return
+        out = np.zeros((nrois, nrois))
+	for k1, k2 in lib.allpairs(range(nrois)):
+	    coef = getter(corrfn(ts[k1], ts[k2]))
+	    out[k2,k1] = coef
+	pl.figure();
+	ax = pl.imshow(out, aspect='equal', interpolation='nearest')
+	ticks = np.arange(nrois)
+	pl.setp(ax.axes, xticks=ticks, yticks=ticks),
+	pl.setp(ax.axes, xticklabels=rois, yticklabels=rois)
+	pl.colorbar()
+	return out
 
     def show_xwt(self, **kwargs):
-        for p in lib.allpairs0(filter(self.isCircleROI, self.roi_objs.keys())):
+        for p in lib.allpairs(filter(self.isCircleROI, self.roi_objs.keys())):
             self.show_xwt_roi(*p,**kwargs)
 
 
